import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'linkv_rtm_callback.dart';
import 'linkv_api_defines.dart';
import 'linkv_video_callback.dart';

class RtmFlutterPlugin {
  static const MethodChannel _channel =
      const MethodChannel('rtm_flutter_plugin');

  static LinkVVideoCallback _videoCallback;
  static LVIMEventCallback _rtmCallback;

  // 标识已读消息自定义类型。
  static const String TYPE_MESSAGE_HAD_READ = "TYPE_MESSAGE_HAD_READ";
  static const String TYPE_MESSAGE_EVENT = "flag_event";

  /// SDK 内部版本号，汇报给 linkv 时可以提供该版本号
  static Future<String> getSdkVersion() async {
    final String version = await _channel.invokeMethod("getSdkVersion");
    return version;
  }

  /// SDK 鉴权
  /// @param appId 向LinkV申请的appId
  /// @param appSign SDK签名字符串
  /// @param isTestEnv  YES 为使用测试环境，NO 为不使用测试环境（默认为 NO）
  /// @param isInternationalEnv YES 为国际版本，NO 为国内版本（默认为 NO）
  static Future<int> initSDK(String appId, String appSign, bool isDebugEnv,
      bool isInternationalEnv) async {
    final int result = await _channel.invokeMethod("initSDK", {
      'rtcAppID': appId,
      'rtcAppKey': appSign,
      'imAppID': appId,
      'imAppKey': appSign,
      'sdkType': 3,
      'isDebugEnv': isDebugEnv,
      'isInternationalEnv': isInternationalEnv
    });
    return result;
  }

  // 设置是否打开日志
  static void setLogOpen(bool isOpen) {
    _channel.invokeMethod("setLogOpen", {"isOpen": isOpen});
  }

  /// 推流参数配置
  ///
  /// @param videoConfigLevel 推流参数预设值
  static void setVideoConfig(int videoConfigLevel) async {
    await _channel
        .invokeMethod("setVideoConfig", {"videoConfigLevel": videoConfigLevel});
  }

  /// 登录，请使用自己的用户账号系统，确保userId唯一标识用户。
  /// @param 用户ID
  /// @return 0代表登录成功，-6代表已经是登录状态，其它代表失败。
  static Future<int> loginUser(String userId) async {
    return await _channel.invokeMethod("loginUser", {"userId": userId});
  }

  static void setIMToken(String uid, String token) async {
    if (token == null || token.length == 0 || uid == null || uid.length == 0)
      return;

    _channel.invokeMethod("setIMToken", {"userId": uid, "imToken": token});
  }

  /// 发送私信消息
  ///
  /// @param sub          媒体类型
  /// @param tid          接收者ID
  /// @param type         消息的自定义扩展类型，可作为消息标识，但不能传空或者空串。
  /// @param content      消息内容
  /// SendMessageResult的字段eCode返回0为成功，其它为错误码
  static Future<SendMessageResult> sendPrivateMessage(
      MessageSubType sub, String tid, String type, String content) async {
    Map map = await _channel.invokeMethod("sendPrivateMessage", {
      "sub": sub.index,
      "tid": tid,
      "type": type,
      "content": content,
    });
    // 将Msg字段的值转成LVIMMsg对象返回
    return SendMessageResult.convertFromMap(map);
  }

  /// 发送消息，主要用于失败重发
  static Future<SendMessageResult> sendMessage(LVIMMsg msg) async {
    Map map = await _channel.invokeMethod("sendMessage",{
      "cmdType": msg.cmdType,
      "fromID": msg.fromID,
      "sub": msg.subType,
      "tid": msg.toID,
      "type": msg.msgType,
      "content": msg.msgContent,
      "cmsgid": msg.cmsgid
    });
    return SendMessageResult.convertFromMap(map);
  }

  /// 发送房间消息
  ///
  /// @param targetId   房间Id
  /// @param msgContent 消息内容
  /// @param listener   发送结果监听
  /// SendMessageResult的字段eCode返回0为成功，其它为错误码
  static Future<SendMessageResult> sendRoomMessage(String targetId, String msgContent) async {
    Map map = await _channel.invokeMethod("sendRoomMessage", {
      "targetId": targetId,
      "msgContent": msgContent,
    });
    // 将Msg字段的值转成LVIMMsg对象返回
    return SendMessageResult.convertFromMap(map);
  }

  /// 发送某条消息已读的消息
  ///
  /// @param tid          接收者ID
  /// @param messageSendTime    已读的最后一条消息的发送时间戳
  /// SendMessageResult的字段eCode返回0为成功，其它为错误码
  static Future<SendMessageResult> sendMessageHasRead(String tid, int messageSendTime) async {
//    return await sendEventMessage(tid, TYPE_MESSAGE_HAD_READ, messageSendTime.toString());
    return await sendPrivateMessage(MessageSubType.IM_SUBTYPE_TEXT, tid, TYPE_MESSAGE_HAD_READ, messageSendTime.toString());
  }

  /// 发送事件消息
  ///
  /// @param tid          接收者ID
  /// @param type         消息的自定义扩展类型，可作为消息标识，但不能传空或者空串。
  /// @param content      消息内容
  /// SendMessageResult的字段eCode返回0为成功，其它为错误码
  static Future<SendMessageResult> sendEventMessage(String tid, String type, String content) async {
//    Map map = await _channel.invokeMethod("sendEventMessage", {
//      "tid": tid,
//      "type": type,
//      "content": content,
//    });
//    // 将Msg字段的值转成LVIMMsg对象返回
//    return SendMessageResult.convertFromMap(map);
    return await sendPrivateMessage(MessageSubType.IM_SUBTYPE_TEXT, tid, TYPE_MESSAGE_EVENT, content);
  }

  /// 删除本地私信会话记录
  /// @param uid 用户id
  /// 成功返回删除条数, 失败返回<0的值
  static Future<int> deleteLocalPrivateSession(String uid) async {
    if (uid == null) return 0;
    return await _channel.invokeMethod("deleteLocalPrivateSession", {
      "uid": uid,
    });
  }

  /// 把某个会话中所有未读消息标记为已读
  /// @param uid 用户id
  /// 成功返回处理条数, 否则返回<0的值
  static Future<int> clearPrivateSessionUnreadMsg(String uid) async {
    if (uid == null) return 0;

    return await _channel.invokeMethod("clearPrivateSessionUnreadMsg", {
      "uid": uid,
    });
  }

  /// 把所有收到的私信未读消息标记为已读
  /// @param uid 用户id
  /// 成功返回处理条数, 否则返回<0的值
  static Future<int> clearPrivateAllUnreadMsg() async {
    return await _channel.invokeMethod("clearPrivateAllUnreadMsg");
  }

  /// 把某个会话中小于等于stime的所有收到的未读消息标记为已读
  /// @param uid   目标的id
  /// @param stime 消息的服务端时间戳
  /// 成功返回处理条数, 否则返回<0的值
  static Future<int> updatePrivateMsgAsReadByStime(String uid, int stime) async {
    if (uid == null || stime <= 0) return 0;
    return await _channel.invokeMethod("updatePrivateMsgAsReadByStime", {
      "uid": uid,
      "stime": stime
    });
  }

  /// 查询本地私信消息历史记录【DESC，距离当前时间最近的一条消息会在第一位】
  /// @param uid   目标的id
  /// @param dbid  消息的数据库id —— 传0表示从最近的一条消息开始查询
  /// @param limit 限制结果条数
  static Future<List<LVIMMsg>> queryLocalPrivateHistoryMessage(String uid, int dbid, int limit) async {
    if (uid == null || dbid < 0 || limit < 0) return null;

    Map map = await _channel.invokeMethod("queryLocalPrivateHistoryMessage", {
      "uid": uid,
      "dbid": dbid,
      "limit": limit,
    });

    if (map["result"] <= 0 || map["msgs"] == null) return null;

    List<LVIMMsg> list = new List();
    for (Map msgMap in map["msgs"]) {
      list.add(LVIMMsg.convertFromMap(msgMap));
    }

    return list;
  }

  /// 获取消息会话列表
  static Future<List> querySessionList() async {
    Map map = await _channel.invokeMethod("querySessionList");

    if (map["result"] <= 0 || map["sessions"] == null) return null;

    List<LVIMSession> list = new List();
    for (Map sessionMap in map["sessions"]) {
      LVIMSession session = new LVIMSession();
      session.unreadCount = sessionMap["unreadCount"];
      session.lastMsg = LVIMMsg.convertFromMap(sessionMap);
      list.add(session);
    }
    return list;
  }

  /// 卸载 SDK，销毁单例对象资源
  static void unInitSDK() async {
    await _channel.invokeMethod("unInitSDK");
  }

  /// 设置SDK事件监听
  static void setIMEventCallback(LVIMEventCallback callback) {
    _rtmCallback = callback;
    _channel.setMethodCallHandler(_handleVideoCallback);
  }

  /// 设置房间事件监听
  static void setVideoCallback(LinkVVideoCallback callback) {
    _videoCallback = callback;
    _channel.setMethodCallHandler(_handleVideoCallback);
  }

  /// 加入房间（音视频和聊天）
  /// @param roomId 房间 ID
  /// @param userId 用户 ID
  /// @param isHost 是否是主持人
  /// @param isOnlyAudio 是否以音频模式加入
  static void loginRoom(
      String roomId, String userId, bool isHost, bool isAudioOnly) async {
    loginRoomWithCallback(roomId, userId, isHost, isAudioOnly, null);
  }

  static void loginRoomWithCallback(
      String roomId, String userId, bool isHost, bool isAudioOnly, LinkVVideoCallback callback) async {
    if (callback != null) {
      setVideoCallback(callback);
    }

    await _channel.invokeMethod("loginRoom", {
      "roomId": roomId,
      "userId": userId,
      "isHost": isHost,
      "isAudioOnly": isAudioOnly
    });
  }

  static Future<Map> requestDebugToken(String userId, String username) async {
    if (userId == null || userId.length == 0) return null;
    return await _channel.invokeMethod(
        "requestDebugToken", {"userId": userId, "username": username});
  }

  /// 设置推流时数据统计信息回调的频率，回调方法请看 OnPublishQualityUpdate 回调
  /// @param seconds 回调频率
  static void setPublishQualityMonitorCycle(int seconds) async {
    await _channel
        .invokeMethod("setPublishQualityMonitorCycle", {'seconds': seconds});
  }

  /// 发布音视频数据（推流），如果登录房间时 isOnlyAudio 为 YES 则只发送音频数据，视频数据不发送，推流状态的变更通过 OnPublishStateUpdate: 回调
  static void startPublishing() async {
    await _channel.invokeMethod("startPublishing");
  }

  /// 设置拉流时数据统计信息回调的频率，回调方法请看 OnPlayQualityUpate 回调
  /// @param seconds 回调频率
  static void setPlayQualityMonitor(int seconds) async {
    await _channel.invokeMethod("setPlayQualityMonitor", {"seconds": seconds});
  }

  /// 开始播放一路音视频流，播放状态的变更通过 OnPlayStateUpdate:userId: 回调
  /// @param isZOrderMediaOverlay surfaceView的层级
  /// @param userId 要播放的音视频流用户 ID
  static void startPlayingStream(
      String userId, bool isZOrderMediaOverlay) async {
    if (userId == null || userId.length == 0) return;

    await _channel.invokeMethod("startPlayingStream",
        {"streamId": userId, "isZOrderMediaOverlay": isZOrderMediaOverlay});
  }

  /// 停止播放一路音视频流
  /// @param userId 要停止播放的音视频流用户 ID
  static void stopPlayingStream(String userId) async {
    if (userId == null || userId.length == 0) return;

    await _channel.invokeMethod("stopPlayingStream", {"streamId": userId});
  }

  /// 退出房间
  static void logoutRoom() async {
    await _channel.invokeMethod("logoutRoom");
  }

  /// 停止视频采集
  static void stopCapture() async {
    await _channel.invokeMethod("stopCapture");
  }

  /// 开始视频采集
  static void startCapture() async {
    await _channel.invokeMethod("startCapture");
  }

  /// 停止发布音视频数据
  static void stopPublishStream() async {
    await _channel.invokeMethod("stopPublishStream");
  }

  /// 切换摄像头
  /// @param position 摄像头位置
  static void switchCamera(bool isFrontCamera) async {
    await _channel
        .invokeMethod("switchCamera", {"isFrontCamera": isFrontCamera});
  }

  /// 设置视频编码参数
  /// @param config 视频分辨率
  /// @discussion 注：如果后面设置的视频采集分辨率大于之前设置的分辨率，摄像头可能会进行重置操作
  static void setAVConfig(LVAVConfig config) async {
    await _channel.invokeMethod("setAVConfig", {"config": config.index});
  }

  /// 静音推流
  /// @param isMute 是否静音
  static void setMuteOutput(bool isMute) async {
    await _channel.invokeMethod("setMuteOutput", {"isMute": isMute});
  }

  /// IM 鉴权是否成功
  static Future<bool> isIMAuthed() async {
    return await _channel.invokeMethod("isIMAuthed");
  }

  /// IM 是否登录成功
  static Future<bool> isIMLoginSucceed() async {
    return await _channel.invokeMethod("isIMLoginSucceed");
  }

  /// 设置美颜
  /// @param beauty 目前包含：美颜、明亮度、饱和度
  /// @param level 取值范围为[0~1]，关闭美颜传-1
  static void setBeautyLevel(LVBeauty beauty, double level) async {
    await _channel.invokeMethod(
        "setBeautyLevel", {"beauty": beauty.index, "level": level});
  }

  // MARK: - Video Callback
  static Future<dynamic> _handleVideoCallback(MethodCall call) {
    if (call.method != "onPublisherQualityUpdate" &&
        call.method != "onRemoteQualityUpdate" &&
        call.method != "onVideoSizeChanged" &&
        call.method != "onQueryIMToken") {
      print("RtmFlutterPlugin call = $call  = ${call.method}");
    }
    if (call.method == "onRemoteStreamAdd") {
      String userId = call.arguments["userId"];
      _videoCallback?.onAddRemoter(userId);
    } else if (call.method == "onRemoteStreamEnd") {
      _videoCallback?.onDeleteRemoter(call.arguments["userId"]);
    } else if (call.method == "onRoomDisconnect") {
      _videoCallback?.onRoomDisconnect(call.arguments["state"]);
    } else if (call.method == "onRoomConnected") {
      _videoCallback?.onRoomConnected();
    } else if (call.method == "onPublisherQualityUpdate") {
      LVVideoStatistic videoStatistic =
          LVVideoStatistic.convertFromMap(call.arguments);
      _videoCallback?.onPublishQualityUpdate(videoStatistic);
    } else if (call.method == "onRemoteQualityUpdate") {
      LVVideoStatistic videoStatistic =
          LVVideoStatistic.convertFromMap(call.arguments);
      _videoCallback?.onPlayQualityUpdate(
          videoStatistic, call.arguments["streamID"]);
    } else if (call.method == "onVideoSizeChanged") {
      _videoCallback?.onVideoSizeChanged(call.arguments["streamID"],
          call.arguments["width"], call.arguments["height"]);
    } else if (call.method == "onRoomMessageReceive") {
      LVIMMsg msg = LVIMMsg.convertFromMap(call.arguments);
      print("RtmFlutterPlugin call msg fromid = " +
          msg.fromID +
          "   content " +
          msg.msgContent.toString());
      _videoCallback?.onRoomMessageReceive(msg);
    } else if (call.method == "onPublishStateUpdate") {
      _videoCallback?.onPublishStateUpdate(call.arguments["state"]);
    } else if (call.method == "onPlayStateUpdate") {
      _videoCallback?.onPlayStateUpdate(
          call.arguments["state"], call.arguments["streamId"]);
    } else if (call.method == "onIMMessageReceive") {
      LVIMMsg msg = LVIMMsg.convertFromMap(call.arguments);
      print("RtmFlutterPlugin call msg fromid = " +
          msg.fromID +
          "   content " +
          msg.msgContent.toString());
      if (msg.msgType == TYPE_MESSAGE_HAD_READ) {
        // 收到已读消息类型
        _rtmCallback.onReadMessageReceive(msg, int.parse(msg.msgContent));
      } else if(msg.msgType == TYPE_MESSAGE_EVENT) {
        // 收到事件类型消息。
        _rtmCallback.onEventMessageReceive(msg);
      } else {
        _rtmCallback?.onIMMessageReceive(msg);
      }
    } else if (call.method == "onQueryIMToken") {
      // IM Auth
      _rtmCallback?.onQueryIMToken();
    } else if (call.method == "onIMAuthFailed") {
      String uid = call.arguments["uid"];
      String token = call.arguments["token"];
      int ecode = call.arguments["ecode"];
      int rcode = call.arguments["rcode"];
      bool isTokenExpired = call.arguments["isTokenExpired"];
      _rtmCallback?.onIMAuthFailed(uid, token, ecode, rcode, isTokenExpired);
    } else if (call.method == "onIMTokenExpired") {
      String uid = call.arguments["uid"];
      String token = call.arguments["token"];
      _rtmCallback?.onIMTokenExpired(uid, token);
    } else if (call.method == "onIMAuthSucceed") {
      int unReadMsgSize = call.arguments["unReadMsgSize"];
      String token = call.arguments["token"];
      String uid = call.arguments["uid"];
      _rtmCallback?.onIMAuthSucceed(uid, token, unReadMsgSize);
    }

    return Future.value(true);
  }
}
